import redis
from fastapi import Request
from kitnet.lib.netdb import PostgreSQL

from model.dhcp_leases_model import DhcpLeaseDB
from model.settings import settings

db = PostgreSQL()


def get_db(request: Request):
    if hasattr(request.state, 'db'):
        return request.state.db

    database = db
    request.state.db = database
    return database


def get_conn(request: Request):
    if hasattr(request.state, 'db_conn'):
        return request.state.db_conn

    db_connection = get_db(request).connect()
    query = "SET search_path TO netadmin, public;"
    db.execute(db_connection, query)
    db_connection.commit()
    request.state.db_conn = db_connection

    # Register callbacks to ensure transactions are properly committed after request is over
    request.state.success_callbacks.append(db_connection.commit)
    request.state.error_callbacks.append(db_connection.rollback)
    request.state.common_callbacks.append(db_connection.close)
    return db_connection


def get_dhcp_lease_db(request: Request):
    if hasattr(request.state, 'dhcp_lease_db'):
        return request.state.dhcp_lease_db

    dhcp_lease_db = DhcpLeaseDB(database=settings.minidhcpvs_db,
                                user=settings.minidhcpvs_user,
                                host=settings.minidhcpvs_host,
                                port=settings.minidhcpvs_port,
                                password=settings.minidhcpvs_password,
                                )

    request.state.common_callbacks.append(dhcp_lease_db.connection.close)
    request.state.dhcp_lease_db = dhcp_lease_db
    return dhcp_lease_db


def get_macfinder_redis(request: Request):
    if hasattr(request.state, 'macfinder_redis'):
        return request.state.macfinder_redis

    r = redis.from_url(settings.redis_url, db=settings.macfinder_redis_db_index)
    # TODO: debug this
    # request.state.common_callbacks.append(r.close)

    request.state.macfinder_redis = r
    return request.state.macfinder_redis
